c
c---------------------------------------------------------------------------------
c
      subroutine calcrot (logrot)
c
c-----Euler angles are a means of representing the spatial orientation
c     of any frame of the space as a composition of rotations from a reference
c     frame. The definition is static. The intersection of the xy and the XY
c     coordinate planes is called the line of nodes (N).
c     
c     * alpha is the angle between the x-axis and the line of nodes.
c     * beta is the angle between the z-axis and the Z-axis.
c     * gamma is the angle between the line of nodes and the X-axis.
c     coordinate systems ====> (x,y,z) ; (X,Y,Z)
c     * alpha and gamm range from 0 to 2*pi radians.
c     * beta ranges from 0 to pi radians.
c
c     The former definition can be seen as composition of three rotations
c     around the intrinsic (moving) or extrinsic (fixed) axis in some initial
c     conditions.
c
c     Given two coordinate systems xyz and XYZ with common origin, starting
c     with the axis z and Z overlapping, one can specify the position of the
c     second in terms of the first using three rotations with angles alpha, beta,
c     gamma in three ways equivalent to the former definition, as follows:
c
c     * Mixed axes of rotation - The xyz system is fixed while the XYZ
c       system rotates. Start with the rotating XYZ system coinciding with
c       the fixed xyz system. Perform the first rotation around z, the
c       second around the line of nodes N and the third around Z. In this
c       way we will reach the final frame starting from the initial one.
c
c     * Moving axes of rotation (See Tait-Bryan angles) - The XYZ system is
c       fixed while the xyz system rotates. If we start with the xyz
c       system coinciding with the XYZ system we can perform the same
c       rotations as before using only rotations around the moving axis.
c       - Rotate the xyz-system about the z-axis by alpha. The x-axis now
c         lies on the line of nodes.
c       - Rotate the xyz-system again about the now rotated x-axis by
c         beta. The z-axis is now in its final orientation, and the X-axis remains
c         on the line of nodes.
c       - Rotate the xyz-system a third time about the new z-axis by
c         gamma.
c       This is the convention described in Goldstein.
c
c     * Fixed axes of rotation - The xyz system is fixed while the XYZ
c       system rotates. Start with the rotating XYZ system coinciding with
c       the fixed xyz system.
c       - Rotate the XYZ-system about the z-axis by gamma. The X-axis is
c         now at angle gamma with respect to the x-axis.
c       - Rotate the XYZ-system again about the x-axis by beta. The
c         Z-axis is now at angle beta with respect to the z-axis.
c       - Rotate the XYZ-system a third time about the z-axis by α.
c
c     These three definitions are equivalent.
c
c     We call [\mathbf{R}] to the matrix that represents the coordinates of
c     the referred frame with respect to the reference frame. This matrix is 
c     equivalent to the composition of three rotations. Therefore:

c     [\mathbf{R}] = \begin{bmatrix} \cos \gamma & \sin \gamma & 0 \\
c     -\sin \gamma & \cos \gamma & 0 \\ 0 & 0 & 1 \end{bmatrix}
c     \begin{bmatrix} 1 & 0 & 0 \\ 0 & \cos \beta & \sin \beta \\ 0 & -\sin
c     \beta & \cos \beta \end{bmatrix} \begin{bmatrix} \cos \alpha & \sin
c      \alpha & 0 \\ -\sin \alpha & \cos \alpha & 0 \\ 0 & 0 & 1 \end{bmatrix}
c     Notice that independently each matrix refers to a rotation around an
c     axis, but when they are multiplied,
c     * The inner (rightmost) matrix refers to a rotation around Z
c     * The outer (leftmost) matrix refers a to rotation around the
c       "fixed" axis z
c     * The one in the middle represents a rotation around the line of
c       nodes.

c----------------------------------------------------------------------------------
c
      use datos, only : rotop, neq1, faterr
      use libreria,only : error
      implicit real*8 (a-h,o-z)
      logical :: logrot
c.....LOCAL VARIABLES
      real*8 :: rzz2, sinbeta
      
      if (logrot) then
       rotop(1,1,neq1)=1d0;rotop(2,2,neq1)=1d0;rotop(3,3,neq1)=-1d0
       rotop(1,2,neq1)=0d0;rotop(1,3,neq1)=0d0
       rotop(2,1,neq1)=0d0;rotop(2,3,neq1)=0d0
       rotop(3,1,neq1)=0d0;rotop(3,2,neq1)=0d0
      else
c
c-----Definition of sin(beta). We have selected the positive root
c     sin(beta) = sqrt(1-rotop(3,3,neq1)**2). Also,
c     note that sin(beta) must belong to [0,1] and, then, 
c     beta must belong to [0, pi]
c       
       rzz2 = rotop(3,3,neq1)*rotop(3,3,neq1)
       sinbeta = sqrt (1d0 - rzz2)
       if (sinbeta.gt.1d0.or.sinbeta.lt.0d0)
     &  call error ('calcrot', 'Beta Euler angle does not belong to
     &  [0,pi]', faterr)
c
c-----We have selected gamma Euler angle equal to 0 radians. Then, 
c     cos(gamma)= 1 and sin(gamma)= 0. Moreover,
c     cos(alpha)= -rotop(3,2,neq1)/sin(beta) and
c     sin(alpha)= rotop(3,1,neq1)/sin(beta).
c     
c
c-----Obtain rotop matrix
c
       rotop(1,1,neq1) = -rotop(3,2,neq1)/sinbeta
       rotop(1,2,neq1) = rotop(3,1,neq1)/sinbeta
       rotop(1,3,neq1) = 0d0
       rotop(2,1,neq1) = -rotop(3,1,neq1)*rotop(3,3,neq1)/sinbeta
       rotop(2,2,neq1) = -rotop(3,2,neq1)*rotop(3,3,neq1)/sinbeta
       rotop(2,3,neq1) = sinbeta
      endif
c
c-----End subroutine
c
      end subroutine
