       module libreria
       implicit real*8 (a-h,o-z)
       contains
c
c.....getopt - parses the command line for arguments, flags and options.
c
      logical function getopt (args, largs, nargs,
     &                         flgs, lflgs, nflgs,
     &                         opts, lopts, nopts, optarg)
c     The routine returns .false. if an error occurs, .true. otherwise.
c
c     Argument is any token in the commmand line that doesn't begin with
c     '-'. A standalone '-' will be considered a special argument, used
c     to skip an arg without filling it.
c
c     Flag is any char of a token that starts with a '-'.
c
c     Option is a flag that accepts an argument, which must be the token
c     following the option.
c
c     A '--' in the input line signals the end of options and flags (in
c     this way, arguments beginning with '-' are allowed).
c
c..Input parameters:
c
c  nargs ......... maximum number of allowed arguments.
c  nflgs ......... number of valid flags.
c  flgs .......... list of valid flags.
c  nopts ......... number of valid options.
c  opts .......... list of valid options.
c
c..Output parameters:
c
c  args() ........ returns the values of the specified arguments.
c  largs() ....... returns .true. on each specified argument (that is,
c                  '-' will increase the count of arguments but with
c                  a .false. in lflags).
c  lflgs() ....... returns a .true. in the position of the flags
c                  actually found in the command line.
c  lopts() ....... returns .true. in the position of used options.
c  optarg() ...... contains the arguments associated to the options
c                  actually found.
c
c..Example:
c
c  Given a program that expects for the next entrance:
c
c       myprog [input] [output] [-abcde] [-o arg1] [-s arg2]
c
c  where square braces [] contain optional arguments, the main program
c  may use the next call to getopt to parse the command line:
c
c       nargs = 2
c       nflgs = 5
c       flgs = 'abcde'
c       nopts = 2
c       opts = 'os'
c       if (.not. getopt (args, largs, nargs, flgs, lflgs, nflgs,
c      &            opts, lopts, nopts, optarg)) then
c          --- Something went wrong ---
c       endif
c
c PARAMETERS:
c
      integer           nargs, nflgs, nopts
      logical           largs(*), lflgs(*), lopts(*)
      character*(*)     args(*), optarg(*)
      character*(*)     flgs, opts
c
c LOCAL VARIABLES:
c
      integer           leng, iargc
      integer           iargs, iflgs, iopts
      integer           ntokens, itoken, ltoken, lp
      character*(128)   token
      logical           endopts
c
c.....get the number of tokens in the command line:
c
      iargs = 0
      ntokens = iargc ()
      if (ntokens.lt.0) then
        getopt = .false.
        return
      endif
c
c.....loop over tokens:
c
      endopts = .false.
      itoken = 1
      do while (itoken.le.ntokens)
        call getarg (itoken, token)
        itoken = itoken + 1
        ltoken = leng (token)
c
c.....is it an arg? (or we found the end of options)
c
        if (token(1:1).ne.'-' .or. endopts) then
          if (iargs.lt.nargs) then
            iargs = iargs + 1
            args(iargs) = token(1:ltoken)
            largs(iargs) = .true.
          else
            write (0,*) 'getopt: too many arguments'
            getopt = .false.
            return
          endif
c
c.....dummy argument:
c
        else if (ltoken.eq.1) then
          if (iargs.lt.nargs) then
            iargs = iargs + 1
            args(iargs) = token(1:ltoken)
            largs(iargs) = .false.
          else
            write (0,*) 'getopt: too many arguments'
            getopt = .false.
            return
          endif
c
c.....end of options:
c
        else if (token(1:ltoken).eq.'--') then
          endopts = .true.
c
c.....everything else are flags and options:
c
        else
c
c.....loop for flags:
c
          do lp = 2, ltoken-1
            iflgs = 1
            do while (iflgs.lt.nflgs .and.
     &                token(lp:lp).ne.flgs(iflgs:iflgs))
              iflgs = iflgs + 1
            enddo
            if (iflgs.eq.nflgs .and.
     &          token(lp:lp).ne.flgs(iflgs:iflgs)) then
              write (0,*)
     &          'getopt: -'//token(lp:lp)//' is not a valid flag'
              getopt = .false.
              return
            endif
            lflgs(iflgs) = .true.
          enddo
c
c.....last one can be a flag or an option:
c
          lp = ltoken
          iflgs = 1
          do while (iflgs.lt.nflgs .and.
     &              token(lp:lp).ne.flgs(iflgs:iflgs))
            iflgs = iflgs + 1
          enddo
c
c.....it is a flag:
c
          if (iflgs.lt.nflgs .or.
     &        token(lp:lp).eq.flgs(iflgs:iflgs)) then
            lflgs(iflgs) = .true.
c
c.....not a flag:
c
          else
            iopts = 1
            do while (iopts.lt.nopts .and.
     &                token(lp:lp).ne.opts(iopts:iopts))
              iopts = iopts + 1
            enddo
c
c.....it is an option:
c
            if (iopts.ne.nopts .or.
     &          token(lp:lp).eq.opts(iopts:iopts)) then
              if (itoken.le.ntokens) then
                lopts(iopts) = .true.
                call getarg (itoken, token)
                itoken = itoken + 1
                ltoken = leng (token)
                optarg(iopts) = token(1:ltoken)
              else
                write (0,*)
     &             'getopt: missing argument in option -'//token(lp:lp)
                getopt = .false.
                return
              endif
c
c.....not an option:
c
            else
              write (0,*)
     &          'getopt: -'//token(lp:lp)//' is not a valid flag'
              getopt = .false.
              return
            endif
          endif
        endif
      enddo
c
c.....end of parsing ok:
c
      getopt = .true.
      return
      end function
c
c ----------------------------------------------------------------------
c
*mdc*if lahey vms
*      integer function iargc ()
*c
*c.....iargc: substitute function for unix's library one. Returns the
*c     number of tokens in the command line; tokens are separated by
*c     blanks; in addition, everything between double quotes is
*c     considered in the same token
*c
*      include           'error.inc'
*      character*(1)     blank,null,dquote
*      parameter ( blank=char(32) , null=char(0) , dquote=char(34) )
*      integer           leng, length, lp, argc
*      logical           done
*      parameter ( margs = 16 )
*      character*(128)   line, token(0:margs)
*      common /commline/ length, argc, line, token
*c
*c.....get command line
*c
*mdc*endif
*mdc*if lahey
*      call getcl(line)
*      length=leng(line)
*mdc*elsif vms
*      include           '($clidef)'
*      call cli$get_value ('$line',line,length)
*mdc*endif
*mdc*if lahey vms
*c
*c.....skip initial spaces
*c
*      argc = 0
*      lp = 1
*      done = .false.
*      do while (.not.done .and. lp.le.length)
*        if (line(lp:lp).ne.blank) then
*          done = .true.
*        else
*          lp = lp + 1
*        endif
*      enddo
*      if (.not.done) then
*        call error ('iargc', 'empty command line', warning)
*        iargc = -1
*        return
*      endif
*c
*c.....skip command name
*c
*      done = .false.
*      i = lp
*      do while (.not.done .and. lp.le.length)
*        if (line(lp:lp).eq.blank) then
*          done = .true.
*        else
*          lp = lp + 1
*        endif
*      enddo
*      token(argc) = line(i:lp-1)
*c
*c.....no arguments
*c
*      if (.not.done) then
*        iargc = 0
*        return
*      endif
*c
*c.....loop over tokens
*c
*      do while (lp.le.length)
*c
*c.....skip blanks
*c
*        done = .false.
*        do while (.not.done .and. lp.le.length)
*          if (line(lp:lp).ne.blank) then
*            done = .true.
*          else
*            lp = lp + 1
*          endif
*        enddo
*c
*c.....end of command line
*c
*        if (.not.done) then
*          iargc = argc
*          return
*        endif
*c
*c.....get next token
*c
*        argc = argc + 1
*        if (argc.gt.margs)
*     &    call error ('iargc', 'too many tokens',faterr)
*        token(argc) = null
*        i = 1
*  10    continue
*        done = .false.
*        do while (.not.done .and. lp.le.length)
*          if (line(lp:lp).eq.blank .or. line(lp:lp).eq.dquote) then
*            done = .true.
*          else
*            token(argc)(i:i) = line(lp:lp)
*            i = i + 1
*            lp = lp + 1
*          endif
*        enddo
*c
*c.....end of command line
*c
*        if (.not.done) then
*          iargc = argc
*          return
*        endif
*c
*c.....token in double quotes; the goto takes into account abc" d "efg
*c     as one token
*c
*        if (line(lp:lp).eq.dquote) then
*          if (lp.eq.length) then
*            write (0,*) 'iargc: unmatched '//dquote
*            iargc = -1
*            return
*          endif
*          lp = lp + 1
*          done = .false.
*          do while (.not.done .and. lp.le.length)
*            if (line(lp:lp).eq.dquote) then
*              done = .true.
*            else
*              token(argc)(i:i) = line(lp:lp)
*              i = i + 1
*              lp = lp + 1
*            endif
*          enddo
*          lp = lp + 1
*          if (.not.done) then
*            write (0,*) 'iargc: unmatched '//dquote
*            iargc = -1
*            return
*          endif
*          goto 10
*        endif
*      enddo
*c
*c.....return the number of arguments
*c
*      iargc = argc
*      return
*      end
*mdc*endif
c
c ----------------------------------------------------------------------
c
*mdc*if lahey vms
*      subroutine getarg (itoken, rtoken)
*c
*c.....getarg: substitute function for unix's library one. Returns the
*c     token number itoken in the command line; tokens are separated by
*c     blanks; in addition, everything between double quotes is
*c     considered to be in the same token, after removing the quotes
*c
*      integer           itoken
*      character*(*)     rtoken
*c
*      integer           length, argc
*      character*(1)     null
*      parameter ( null = char(0) )
*      parameter ( margs = 16 )
*      character*(128)   line, token(0:margs)
*      common /commline/ length, argc, line, token
*c
*c.....test for valid itoken
*c
*      if (itoken.lt.0 .or. itoken.gt.argc) then
*        write (0,*) 'getarg: invalid token number ',itoken
*        rtoken = null
*      else
*        rtoken = token(itoken)
*      endif
*      return
*      end
*mdc*endif
c-----------------------------------------------------------------------
c
c
c.....setword - get next word from line at lp and increase lp
c             - a word is defined as any sequence on nonblanks
c             - setword returns .true. if it finds a non-empty word,
c               false otherwise
c
      logical function setword (word, line, lp)
c
      character*(*)     word, line
      character*1       blank,newline,null
      integer           lp
      integer           l, ll
      logical           done
c
      blank = char(32)
      newline = char(10)
      null = char(0)
c
c.....test end of string
c
      l = len(word)
      ll = len(line)
      setword = .false.
      word = null
      if (lp.gt.ll) return
c
c.....skip blanks
c
      done = .false.
      do while (lp.le.ll .and. .not.done)
         done = line(lp:lp).ne.blank
         lp = lp + 1
      enddo
      if (.not.done) return
      lp = lp - 1
c
c.....get the word
c
      done = .false.
      i = lp
      do while (lp.le.ll .and. .not.done)
         done = line(lp:lp).eq.blank .or.
     &          line(lp:lp).eq.null  .or.
     &          line(lp:lp).eq.newline
         lp = lp + 1
      enddo
      if (done) lp = lp - 1
      word = line(i:lp-1)
      setword = .true.
      return
      end function
c
c-----------------------------------------------------------------------
c
c
c.....leng - obtains the length of the string, assuming that the blanks
c     at the end are dummy.
c
      integer function leng (string)
c
      character*(*)  string
      character*(1)  blank, null
      parameter      (blank = ' ')
      null = char(0)
      do 10 i=len(string),1,-1
         if (string(i:i).ne.blank .and. string(i:i).ne.null) then
            leng=i
            return
         endif
 10   continue
      leng=0
      return
      end function
c
c-----------------------------------------------------------------------
c
      logical function setline (u, line, lp)
c
c.....setline - get line from file with unit # u, and set lp to 1
c     this version skips blank lines, reading ahead
c
      character*(*)     line
      integer           u, lp
c
      integer           ll
c
      lp = 1
 10   read (u,'(a)',end=20) line
         ll = leng(line)
      if (ll .eq. 0) goto 10
      setline = .true.
      return
c
c.....end-of-u-file:
c
 20   setline = .false.
      return
      end function
c
c-----------------------------------------------------------------------
c
      subroutine error (routine,message,errortype)
c
       use datos, only : faterr, warning, noerr
c
      character*(*)       routine,message
      integer             errortype
      character*(20)      chtype
c
      if (errortype.eq.faterr) then
         chtype='fatal error'
      else if (errortype.eq.warning) then
         chtype='warning'
      else if (errortype.eq.noerr) then
         chtype='no error'
      else
         chtype='unknown error'
      endif
      write (*,100) chtype(1:leng(chtype)),routine,message
      if (errortype.eq.faterr) stop
c
 100  format (/,1x,'*** '/,1x,'*** ',a,' at ',a,' :'/,
     &          1x,'*** ',a/,1x,'***')
c
      end subroutine
c
c-----------------------------------------------------------------------
c
      logical function setdble (dblevar, line, lp)
c
c.....setdble - get a real number from line and sets dblevar to it.
c
c     If a valid real number is not found, setdble returns .false.
c
      real*8  dblevar
      character*(*)     line
      integer           lp
c
      logical           isdigit, hasdigits
      integer           dsign, dexpo, desig
      real*8  dval, dpow, ten
      parameter (ten=1d1)
c
      integer           istate, none, sign, digit, dot, decim, expo
      integer           esign, edigit, done
      parameter (none=0, sign=1, digit=2, dot=3, decim=4, expo=5,
     &           esign=6, edigit=7, done=8)
c
      character*(1)     ch, blank, newline, null
*mdc*if g77
*mdc*else
*      parameter ( blank = char(32), newline = char(10), null = char(0))
*mdc*endif
c
      isdigit(ch) = ch.ge.'0' .and. ch.le.'9'
*mdc*if g77
      blank = char(32)
      newline = char(10)
      null = char(0)
*mdc*endif
c
c.....we don't know for sure ...
c
      setdble = .false.
c
c.....test for end of string
c
      ll = len (line)
      if (lp.gt.ll) return
c
c.....skip blanks
c
      istate = none
      do while (lp.le.ll .and. istate.eq.none)
         if (line(lp:lp).ne.blank) istate = done
         lp = lp + 1
      enddo
      if (istate.eq.none) return
      lp = lp - 1
c
c.....finite authomaton
c
      i = lp
      istate = none
      hasdigits = .false.
      dsign = 1
      dval  = 0d0
      dpow  = 1d0
      dexpo = 0
      desig = 1
      do while (istate.ne.done .and. i.le.ll)
         if (line(i:i).eq.'+' .or.
     &       line(i:i).eq.'-') then
            if (istate.eq.none) then
               istate = sign
               if (line(i:i).eq.'-') dsign = -1
            else if (istate.eq.expo) then
               istate = esign
               if (line(i:i).eq.'-') desig = -1
            else
               return
            endif
         else if (line(i:i).eq.'.') then
            if (istate.eq.none .or.
     &          istate.eq.sign .or.
     &          istate.eq.digit) then
               istate = dot
            else
               return
            endif
         else if (index('eEdD',line(i:i)).gt.0) then
            if (istate.eq.digit .or.
     &          istate.eq.decim .or.
     &         (istate.eq.dot .and. hasdigits)) then
               istate = expo
            else
               return
            endif
         else if (isdigit(line(i:i))) then
            if (istate.eq.none .or.
     &          istate.eq.sign .or.
     &          istate.eq.digit) then
               istate = digit
               hasdigits = .true.
               dval = ten*dval + ichar(line(i:i)) - ichar('0')
            else if (istate.eq.dot .or.
     &               istate.eq.decim) then
               istate = decim
               dval = ten*dval + ichar(line(i:i)) - ichar('0')
               dpow = ten*dpow
            else if (istate.eq.expo  .or.
     &               istate.eq.esign .or.
     &               istate.eq.edigit) then
               istate = edigit
               dexpo = ten*dexpo + ichar(line(i:i)) - ichar('0')
            else
               return
            endif
         else if (index(blank//','//null//newline,line(i:i)).gt.0) then
            if (istate.eq.digit  .or.
     &          istate.eq.decim  .or.
     &          istate.eq.edigit .or.
     &         (istate.eq.dot .and. hasdigits)) then
               istate = done
            else
               return
            endif
         else
            return
         endif
         i = i + 1
      enddo
c
c.....check for done when the end is reached
c
      if (istate.ne.done .and. i.eq.ll+1) then
         if (istate.eq.digit  .or.
     &       istate.eq.decim  .or.
     &       istate.eq.edigit .or.
     &      (istate.eq.dot .and. hasdigits)) then
            istate = done
         else
            return
         endif
      else if (istate.eq.done) then
         i = i - 1
      endif
c
c.....set the variable if done
c
      if (istate.eq.done) then
         dblevar = (dsign*dval/dpow) * ten**(desig*dexpo)
         lp = i
         setdble = .true.
      endif
c
c.....end
c
      return
      end function

c
c-----------------------------------------------------------------------
c
      character*(16)     function itoa (ival)
c
c.....itoa - convert integer to string
c
      integer           ival
      character*(1)     null
      character*(16)    buffer
      integer           i,n,sign
*mdc*if g77
      null = char(0)
*mdc*else
*      parameter ( null = char(0))
*mdc*endif
c
      sign = 0
      if (ival .eq. 0) then ! ival = 0 
         itoa='0'//null
         return
      elseif (ival .lt. 0) then ! ival < 0 
         sign=-1
      endif
c
      n=abs(ival) ! n = |ival|
      buffer(16:16)=null
      i=16
      do while (n .gt. 0)
         i=i-1
         buffer(i:i)=char(mod(n,10)+ichar('0'))
         n=n/10
      end do
      if (sign .eq. -1) then 
         i=i-1
         buffer(i:i)='-'
      endif
      itoa=buffer(i:16)
      return 
      end function
c
c-----------------------------------------------------------------------
c
      character*(200) function uppcase (word)
c
c.....uppcase - transform alphabetical characters in word to uppercase.
c     word and uppcase return the same value.
c
      character*(*)     word
      integer           despl
c
*mdc*if gnu
      ilowa = ichar('a')
      ilowz = ichar('z')
      iuppA = ichar('A')
      despl = iuppA - ilowa
*mdc*else
*      parameter (ilowa=ichar('a'), ilowz=ichar('z'), iuppA=ichar('A'))
*      parameter (despl=iuppA-ilowa)
*mdc*endif
c
      do 10 i = 1, len(word)
         ich = ichar(word(i:i))
         if (ich.ge.ilowa .and. ich.le.ilowz) then
            word(i:i) = char(ich+despl)
         endif
 10   continue
      uppcase = word
      end function
c
c-----------------------------------------------------------------------
c
      logical function setint (intvar, line, lp)
c
c.....setint - sets intvar to the integer value of the next word in
c     line.
c
c     If the next word is not a proper integer, setint returns .false.
c
c
      integer           intvar, lp
      character*(*)     line
c
      integer           i, ivar
      logical           isdig, done, iread
c
      character*(1)     ch, blank, newline, null
*mdc*if g77
*mdc*else
*      parameter ( blank = char(32), newline = char(10), null = char(0))
*mdc*endif
c
      isdig (ch) = ch.ge.'0' .and. ch.le.'9'
*mdc*if g77
      blank = char(32)
      newline = char(10)
      null = char(0)
*mdc*endif
c
      setint = .false.
      ll = len(line)
      done = .false.
      do while (lp.le.ll .and. .not.done)
         done = line(lp:lp) .ne. blank
         lp = lp + 1
      enddo
      if (.not.done) return
      lp = lp - 1
      i = lp
      if (line(i:i) .eq. '+') then
         isign = 1
         i = i + 1
      else if (line(i:i) .eq. '-') then
         isign = -1
         i = i + 1
      else
         isign = 1
      endif
      if (i.eq.ll+1) return
      ivar = 0
      done = .false.
      iread = .false.
      do while (.not.done .and. i.le.ll)
         if (isdig(line(i:i))) then
            ivar = 10*ivar + ichar(line(i:i)) - ichar('0')
            iread = .true.
         else if (index(blank//','//newline//null,line(i:i)).gt.0) then
            done = .true.
         else
            return
         endif
         i = i + 1
      enddo
      if (done) i = i - 1
      if (iread) then
         intvar = isign * ivar
         lp = i
         setint = .true.
      endif
      return
      end function
c
c-----------------------------------------------------------------------
c
        subroutine bol(rcart,          ! input
     &                 rad,theta,phi)  ! output
c----------------------------------------------------------------------
c
c       FUNCTION : Transforms cartesian coordinates into spherical ones.
c       AUTHORS  : P.Popelier. Modified by Carlos J. F. Solano.
c       INPUT    : rcart(1:3) = cartesian coordinates.
c       OUTPUT:    rad,theta,phi = spherical coordinates in RADIANS.
c       REMARKS : [1] The inverse mapping is made one to one !!
c                     acos() maps from [-1,1] to [0,pi], so no
c                     correction is needed for theta, but for phi
c                     we must have the image [0,2*pi]. This is
c                     done by looking at the sign of "rcart(2)" or y.
c                     So no negative phi value can be obtained !
c                 [2] If x=y=0 then phi is undetermined. It is set to
c                     zero here.
c
c----------------------------------------------------------------------
        use datos, only : pi, faterr
        real*8 , dimension (:) :: rcart(3)
        real*8 :: rad,theta,phi
c.......LOCAL VARIABLES:
        real*8 :: pi2,vsmall,tus,ccc

c--------------------------- initialise ----------------------------
        pi2=2d0*pi
        vsmall=1d-50
c-------------------------------------------------------------------
           rad=sqrt(rcart(1)*rcart(1)+rcart(2)*rcart(2)+
     &              rcart(3)*rcart(3))
           tus=sqrt(rcart(1)*rcart(1)+rcart(2)*rcart(2))

           if(rad.lt.vsmall) then
              rad=0d0
              theta=0d0
              phi=0d0
              return
           endif

           ccc=rcart(3)/rad   ! ccc can never be much bigger than 1
           if(ccc.gt.1.d0) then   ! to make sure that -1<ccc<1
              ccc=1d0
           else if (ccc.lt.-1) then
              ccc=-1d0
           endif
           theta=acos(ccc)

           if(tus.lt.vsmall) then
               phi=0d0
           else
               phi=acos(rcart(1)/tus)  ! image=[0,pi]
          endif

          if(abs(rcart(2)).lt.vsmall) rcart(2)=0d0
          if(rcart(2).lt.0d0) phi=pi2-phi  ! alter-image=[pi,2*pi]
          if(phi.gt.pi2)
     &     call error ('bol', 'phi>2*pi', faterr)
c
        return
        end subroutine
c
c-----------------------------------------------------------------------
c
        subroutine rsh(theta,phi,slm,il0,lmax)
c-------------------------------------------------------------------
C    ***************************************************************
C    * S_{l,m} will be stored as element (l(l+1)+m) of a           *
C    * supervector slm(0:lmax(lmax+2)), and accessed as            *
C    * slm(il0(l)+m)                                               *
C    * il0.......index of the S_{l,0} (il0(l)=l(l+1))              *
C    ***************************************************************
C    
      use datos, only : warning, pi
      integer :: lmax
      real*8 :: theta, phi
      integer, dimension (:) :: il0(0:lmax)
      real*8, dimension (:) :: slm(0:lmax*(lmax+2)) 
c.....LOCAL VARIABLES:
      integer :: il, im, ip, l, m, l1, l2, m2
      real*8 :: ct, st, cp, sp, g, f, cc, ss, tmp
      real*8 :: sum
C    ****************************************************************
C    * PART I: Compute il0 indice for acces to slm supervector      *
C    ****************************************************************
      il0(0)=0
        il=0
        do l=1,lmax
         il=il+l+l
         il0(l)=il
      enddo
C    ****************************************************************
C    * PART II: Computes RSH up to l=lmax for a polar angles        *
C    * (theta,phi): results are stored as a supervector in slm      *
C    ****************************************************************
c.....S_00 is always 1/sqrt(4pi)
      slm(0)=1d0/sqrt(4d0*pi)
c.....Sines and cosines
      ct=cos(theta)
      st=sin(theta)
      cp=cos(phi)
      sp=sin(phi)
c.....Compute \Theta functions, storing them in Slm (m>0)
      do l = 1, lmax
        m = l - 1
        im = il0(m) + m
        il = il0(l) + l
        slm(il) = sqrt(dble(l+l+1)/dble(l+l)) * st * slm(im)
        g = sqrt(dble(l+l+1))
        slm(il-1) = g * ct * slm(im)
        do ll = l+1, lmax
          f = sqrt( dble((ll+ll+1)*(ll+ll-1)) / dble((ll+m)*(ll-m)) )
          slm(il0(ll)+m) = f * (ct*slm(il0(ll-1)+m)
     &                              - slm(il0(ll-2)+m)/g)
          g = f
        enddo
      enddo
c.....Compute \Phi functions and multiply them by \Theta
      cc = sqrt(2d0)
      ss = 0d0
      do m = 1, lmax
        tmp = cc*cp - ss*sp
        ss = ss*cp + cc*sp
        cc = tmp
        m2 = m+m
        ip = il0(m-1)+m
        do l2 = m2, lmax+lmax, 2
          ip = ip + l2
          slm(ip-m2) = slm(ip) * ss
          slm(ip) = slm(ip) * cc
        enddo
      enddo 
C    *****************************************************************
C    * PART III: Test wheter Sum_m S_lm(r)S_lm(r)=(2l+1)/4pi         *
C    *****************************************************************
      do l=0,lmax
         sum=0d0
         do m=-l,l
          sum=sum+slm(il0(l)+m)*slm(il0(l)+m)
         enddo
         sum=sum*4d0*pi/(l+l+1)
         if (abs(sum-1d0).gt.1d-14) then
          call error ('rsh', 'Deviation in RSH gt 1d-14', warning)
          write (*,100) 'slm in l = ',l,'Error = ',sum-1d0
         endif
      enddo 
c
c.....Formats
c
 100  format (2x,a,i5,2x,a,e23.16)  
c
      return
      end subroutine
c
c-----------------------------------------------------------------------
c
        subroutine rsh2(theta,phi,slm,il0,lmax)
c-------------------------------------------------------------------
C    ***************************************************************
C    * S_{l,m} will be stored as element (l(l+1)+m) of a           *
C    * supervector slm(0:lmax(lmax+2)), and accessed as            *
C    * slm(il0(l)+m)                                               *
C    * il0.......index of the S_{l,0} (il0(l)=l(l+1))              *
C    ***************************************************************
C    
      use datos, only : warning, pi
      integer :: lmax
      real*8 :: theta, phi
      integer, dimension (:) :: il0(0:lmax)
      real*8, dimension (:) :: slm(0:lmax*(lmax+2))
c.....LOCAL VARIABLES:
      integer :: il, im, ip, l, m, l1, l2, m2
      real*8 :: ct, st, cp, sp, g, f, cc, ss, tmp
      real*8 :: sum
C    ****************************************************************
C    * PART I: Computes RSH up to l=lmax for a polar angles         *
C    * (theta,phi): results are stored as a supervector in slm      *
C    ****************************************************************
c.....S_00 is always 1/sqrt(4pi)
      slm(0)=1d0/sqrt(4d0*pi)
c.....Sines and cosines
      ct=cos(theta)
      st=sin(theta)
      cp=cos(phi)
      sp=sin(phi)
c.....Compute \Theta functions, storing them in Slm (m>0)
      do l = 1, lmax
        m = l - 1
        im = il0(m) + m
        il = il0(l) + l
        slm(il) = sqrt(dble(l+l+1)/dble(l+l)) * st * slm(im)
        g = sqrt(dble(l+l+1))
        slm(il-1) = g * ct * slm(im)
        do ll = l+1, lmax
          f = sqrt( dble((ll+ll+1)*(ll+ll-1)) / dble((ll+m)*(ll-m)) )
          slm(il0(ll)+m) = f * (ct*slm(il0(ll-1)+m)
     &                              - slm(il0(ll-2)+m)/g)
          g = f
        enddo
      enddo
c.....Compute \Phi functions and multiply them by \Theta
      cc = sqrt(2d0)
      ss = 0d0
      do m = 1, lmax
        tmp = cc*cp - ss*sp
        ss = ss*cp + cc*sp
        cc = tmp
        m2 = m+m
        ip = il0(m-1)+m
        do l2 = m2, lmax+lmax, 2
          ip = ip + l2
          slm(ip-m2) = slm(ip) * ss
          slm(ip) = slm(ip) * cc
        enddo
      enddo
C    *****************************************************************
C    * PART II: Test wheter Sum_m S_lm(r)S_lm(r)=(2l+1)/4pi          *
C    *****************************************************************
      do l=0,lmax
         sum=0d0
         do m=-l,l
          sum=sum+slm(il0(l)+m)*slm(il0(l)+m)
         enddo
         sum=sum*4d0*pi/(l+l+1)
         if (abs(sum-1d0).gt.1d-14) then
          call error ('rsh', 'Deviation in RSH gt 1d-14', warning)
          write (*,100) 'slm in l = ',l,'Error = ',sum-1d0
         endif
      enddo
c
c.....Formats
c
 100  format (2x,a,i5,2x,a,e23.16)
c
      return
      end subroutine
c
c-----------------------------------------------------------------------
c
      real*8 function factorial(number)

      use datos, only: faterr, ntop, a
      integer:: number
c.....LOCAL VARIABLES:
      real*8 :: gammln

      if (number.lt.0)
     & call error ('factorial','Argument is a negative number',
     & faterr)

      if (number.le.ntop) then
         factorial=a(number+1)
      else if (number.le.32) then
         do j=ntop+1,number
          a(j+1)=j*a(j)
       enddo
       ntop=number
       factorial=a(number+1)
      else
         factorial=exp(gammln(number+1d0))
      endif
      end function factorial
c
c-----------------------------------------------------------------------
c

      subroutine gcf(gammcf,a,x,gln)

      use datos, only : faterr
      integer ITMAX
      real*8 a,gammcf,gln,x,EPS,FPMIN
      parameter (ITMAX=100,EPS=3.e-7,FPMIN=1.e-30)
C     USES gammln
      !Returns the incomplete gamma function
      !Q(a,x)=1-P(a,x) evaluated by its continued
      !fraction representation as gammcf. Also returns
      ! ln(gamma_function(a)) as gln.
      !Parameters: ITMAX is the maximum allowed number
      !of iterations; EPS is the relative accuracy;
      !FPMIN is a number near the smallest representable 
      !floating-point number.
      integer i
      real*8 an,b,c,d,del,h,gammln
      gln=gammln(a)
      b=x+1.-a
      c=1./FPMIN
      d=1./b
      h=d
      do 11 i=1,ITMAX
        an=-i*(i-a)
        b=b+2.
        d=an*d+b
        if(abs(d).lt.FPMIN)d=FPMIN
        c=b+an/c
        if(abs(c).lt.FPMIN)c=FPMIN
        d=1./d
        del=d*c
        h=h*del
        if(abs(del-1.).lt.EPS)goto 1
11    continue
      call error('gcf', 'a too large, ITMAX too small in gcf',
     &faterr)

1     gammcf=exp(-x+a*log(x)-gln)*h

      return
      end subroutine
c
c-----------------------------------------------------------------------
c

      subroutine gser(gamser,a,x,gln)

      use datos, only : faterr
      integer ITMAX
      real*8 a,gamser,gln,x,EPS
      parameter (ITMAX=100,EPS=3.e-7)
C     USES gammln
      !Returns the incomplete gamma function
      !P(a,x) evaluated by its serires representation
      !as gamser. Also returns ln(gamma_function(a)) as gln.
      integer n
      real*8 ap,del,sum,gammln
      gln=gammln(a)
      if(x.le.0.)then
        if(x.lt.0.)
     &  call error ('gser', 'x < 0 in gser', faterr)
        gamser=0.
        return
      endif
      ap=a
      sum=1./a
      del=sum
      do 11 n=1,ITMAX
        ap=ap+1.
        del=del*x/ap
        sum=sum+del
        if(abs(del).lt.abs(sum)*EPS)goto 1
11    continue
      call error ('gser','a too large, ITMAX too small in gser',
     &faterr)
1     gamser=sum*exp(-x+a*log(x)-gln)

      return
      end subroutine
c
c-----------------------------------------------------------------------
c

      real*8 function gammln(xx)
      real*8 xx
      !Returns the value ln[gamma_function(xx)] for xx>0.^M
      integer j
      double precision ser,stp,tmp,x,y,cof(6)
      save cof,stp
      data cof,stp/76.18009172947146d0,-86.50532032941677d0,
     &24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
     &-.5395239384953d-5,2.5066282746310005d0/
      x=xx
      y=x
      tmp=x+5.5d0
      tmp=(x+0.5d0)*log(tmp)-tmp
      ser=1.000000000190015d0
      do 11 j=1,6
        y=y+1.d0
        ser=ser+cof(j)/y
11    continue
      gammln=tmp+log(stp*ser/x)

      return
      end function

c
c-----------------------------------------------------------------------
c

      real*8 function gamm(a,x)

      use datos, only: faterr 
c
      real*8 a, x
C     USES gcf,gser
      !Returns the incomplete gamma function^M
      !gamma(a,x). Also gammp is gamma function^M
      !P(a,x)=gamma(a,x)/gamma_function(a)^M
c.....LOCAL VARIABLES:
      real*8 gammp, gammcf,gamser,gln

      if(x.lt.0.d0.or.a.le.0.d0)
     &  call error ('gamm','bad arguments in gamm', faterr)
      if(x.lt.a+1.d0)then
        call gser(gamser,a,x,gln)
        gammp=gamser
      else
        call gcf(gammcf,a,x,gln)
        gammp=1.-gammcf
      endif
      gamm=gammp*exp(gln)

      return
      end function
c
c-----------------------------------------------------------------------
c

      real*8 function lagpol ( n, alpha, x )

      use datos, only : faterr
      integer:: n
      real*8:: alpha, x
c.....LOCAL VARIABLES:
      integer:: i, alpha1, alpha2
      real*8:: const, xc
      real*8, dimension (:):: cx(0:n)

      if (alpha.le.-1.0d0)
     &  call error ('lagpol', 'The input value of alpha must be
     & greater than -1', faterr)

      if (n.lt.0)
     & call error ('lagpol', 'The input value of n is a negative
     & integer', faterr)

       alpha1=int(alpha-0.5d0)
       alpha2=int(alpha+0.5d0)
       const=(2.d0*x)**alpha1
       xc=x**2

       cx(0) = const*factorial(alpha1)/factorial(2*alpha1+1)
       if (n.eq.0) then
        lagpol = cx(0)
        return
       end if

       cx(1) =(1.d0+alpha-xc)*const*factorial(alpha2)
     &        /(factorial(2*alpha2+1))

       do i = 2, n
        cx(i) = ((2.d0*dble(i)+alpha-1.d0-xc)*cx(i-1)-cx(i-2)/4.d0)
     &          /(4.d0*dble(i)*(dble(i)+alpha))
       enddo
       lagpol = cx(i)

       return
       end function

c
c-----------------------------------------------------------------------
c
      subroutine lagpol_0(n, cx)

      use datos, only : faterr
      integer:: n
      real*8, dimension (:):: cx(0:n)

      if (n.lt.0)
     & call error ('lagpol_0', 'The input value of n is a negative
     & integer', faterr)

      cx(0) = 1.d0
      if (n.eq.0) then
       return
      endif
    
      cx(1) = 0.25d0
      do i = 2, n
       cx(i) = (dble(2*i-0.5d0)*cx(i-1)-0.25d0*cx(i-2))
     & /(dble(2*i)*dble(2*i+1))
      enddo

      return
      end subroutine
c
c-----------------------------------------------------------------------
c
      subroutine laguerre_func( n, alpha, x, cxx)

      use datos, only : faterr
      integer:: n
      real*8:: alpha, x, cxx
c.....LOCAL VARIABLES:
      integer:: i
      real*8:: const1, const2, expx, xd
      real*8, dimension (:):: cx(0:n)

      if (alpha.le.-1.0d0)
     &  call error ('laguerre_func', 'The input value of alpha must be
     & greater than -1', faterr)

      if (n.lt.0)
     & call error ('laguerre_func', 'The input value of n is a negative
     & integer', faterr)

      const1=2.d0**(0.5d0)
      const2=2.d0**(1.5d0)
      expx=exp(-x)
      xd=2.d0*x

      cx(0) = xd**(alpha+1.d0)*expx*const1
      cxx = cx(0)
      if (n.eq.0) then
       return
      endif

      cx(1) = (1.d0+alpha-x)*xd**(alpha+2.d0)*const2*expx

      do i = 2, n
       cx(i) = 4.d0*x*(dble(2*i-1)+alpha-x)*cx(i-1)
     & -16.d0*x**2*dble(i-1)*(dble(i-1)+alpha)*cx(i-2)
      enddo
      cxx=cx(n)

      return
      end subroutine
c
c-----------------------------------------------------------------------
c
      real*8 function rgaunt (l1,m1,l2,m2,l3,m3)
c
c.....rgaunt - compute Gaunt coefficient for real spherical harmonics
c
c     The routine evaluates the integral between three real spherical^M
c     harmonics:
c      l1 m1
c     d            = < S      S      S     >
c      l2 m2 l3 m3      l1 m1  l2 m2  l3 m3
c
c     This integral is symmetric with respect to the interchange of
c     functions, i.e. (li,mi) pairs. It is sometimes called Gaunt
c     integral or Gaunt coefficient. This routine calls cgaunt to
c     calculate the \theta part, expecting to receive a Gaunt
c     coefficient between complex spherical harmonics, defined as:
c
c      l1 m1
c     c            = < Y     | Y     | Y     >       (m3=m1-m2)
c      l2 m2 l3 m3      l1 m1   l2 m2   l3 m3
c
         
c
       integer           l1, m1, l2, m2, l3, m3
       integer           m1a, m2a, m3a
       real*8            dvalue, cgaunt
c
c.....Initialize variables
c
       m1a = abs(m1)
       m2a = abs(m2)
       m3a = abs(m3)
c
c.....All m's are zero (case d)
c
       if (m1a+m2a+m3a.eq.0) then
         rgaunt = cgaunt(l3,l1,0,l2,0)
         return
c
c.....Cases e y h
c
       else if (m1*m2*m3.lt.0) then 
           rgaunt=0d0
           return
c
c.....Case f
c
       else if (m1+m2+m3.eq.0.and.m1*m2*m3.eq.0) then
           rgaunt=0d0
           return
c
c.....|m1|=|m2|+|m3|
c
       else if (m1a.eq.m2a+m3a) then
         dvalue = cgaunt(l3,l1,m1a,l2,m2a)
c
c.....|m2|=|m1|+|m3|
c
       else if (m2a.eq.m1a+m3a) then
         dvalue = cgaunt(l3,l2,m2a,l1,m1a)
c
c.....|m3|=|m1|+|m2| 
c     
       else if (m3a.eq.m1a+m2a) then
         dvalue = cgaunt(l1,l3,m3a,l2,m2a)
c
c.....Cases c, g and the others
c
       else
         rgaunt=0d0
         return
       endif
c
c.....All positive or two negative and one positive
c     (cases a and j)
c
      if (m1*m2*m3.gt.0) then
        dvalue = dvalue/sqrt(2d0)
c
c.....If the positive value equals the negatives' sum
c     (case j)
c
        if (m1+m2+m3.eq.0) dvalue = -dvalue
       endif
c
c.....Everything done
c
       rgaunt = dvalue
       return
       end function
c
c-----------------------------------------------------------------------
c

      real*8 function cgaunt (j3, j1, m1, j2, m2)

      use datos, only : pi
c
c.....cgaunt - compute Gaunt coefficient for complex spherical harmonics
c
c.....The routine uses the Gaunt expansion for 3j symbols, returning:
c        j1 m1
c       c                = <j1,m1|j2,m2|j3,(m1-m2)>
c        j2 m2 j3 (m1-m2)
c     where |ji,mi> are (complex) spherical harmonics.
c
c     The routine assumes it is never called with wrong arguments; so,
c     the indices must fulfill:
c
c       |j1-j2| <= j3 <= (j1+j2),         (j1+j2+j3)/2 is integer,
c       |m1| <= j1,      |m2| <= j2,      |m1-m2| <= j3
c
c
      integer           j3, j1, m1, j2, m2
c
      integer           m3, j, k, kmin, kmax, ik
      real*8            lncoef1, coef2, lndumy, sumln
c
c.....Local variables
c
      m3 = m1 - m2
      j = (j1+j2+j3) / 2
c
c.....Gaunt algorithm
c
      lncoef1 = sumln(j1+j2-j3)+sumln(j1-j2+j3)+sumln(j2+j3-j1)+sumln(j)
     &-(sumln(j-j1)+sumln(j-j2)+sumln(j-j3)+sumln(j+j+1))
     &+0.5d0*(sumln(j1+m1)+sumln(j2+m2)+sumln(j3+m3)+sumln(j1-m1)
     &+sumln(j2-m2)+sumln(j3-m3))
      kmin = max(0,j2-j3+m1,j1-j3+m2)
      kmax = min(j1+j2-j3,j1+m1,j2+m2)
      if (mod(kmin+m2+j3+j,2).eq.0) then
        ik = 1
      else
        ik = -1
      endif
      coef2 = 0d0
      do k = kmin, kmax
        lndumy = sumln(k)+sumln(j3-j2-m1+k)+sumln(j3-j1-m2+k)
     &+sumln(j1+j2-j3-k)+sumln(j1+m1-k)+sumln(j2+m2-k)
        coef2 = coef2 + ik *exp(lncoef1 - lndumy)
        ik = -ik
      enddo
      cgaunt = sqrt((j1+j1+1)*(j2+j2+1)*(j3+j3+1)/(4d0*pi))*coef2
c
c.....return
c
      return
      end function
c
c-----------------------------------------------------------------------
c
        real*8 function sumln (j)
        
        use datos, only: faterr, sumtop, lfac
        integer :: j, n

        if (j.lt.0) 
     &   call error ('sumln', 'j is a negative number', faterr)
        if (j.le.sumtop) then
         sumln=lfac(j)
        else if (j.le.100) then
         do n=sumtop+1,j
          lfac(n)=lfac(n-1)+log(dble(n))
         enddo
         sumtop=j
         sumln=lfac(j)
        else   
         sumln=0d0
         do n=2,j0
          sumln=sumln+log(dble(n))
         enddo
        endif

        end function sumln   
c
c-----------------------------------------------------------------------
c
      real*8 function tensor(n1,l1,m1,n2,l2,m2)

      use datos, only : sigm, pi, R, faterr
      integer :: n1, l1, m1
      integer :: n2, l2, m2

c.....LOCAL VARIABLES:
      integer :: lmax, lmin
      integer :: ltot, mtot, ntot, icte
      real*8 :: cte, cte2, numb

      lmax=l1+l2
      lmin=iabs(l1-l2)
      numb=0.d0
      cte=16.d0*pi**(1.5d0)/(sigm**(n1+n1+l1)*sigm**(n2+n2+l2)*R)
      do ltot=lmin,lmax,2
        if (mod(l1+l2+ltot,2).ne.0)
     &   call error ('tensor', 'l1+l2+ltot is odd number', faterr)
        ntot=n1+n2+(l1+l2-ltot)/2
        icte=(l1+l1+l2+l2+ltot)/2
        if (mod(icte,2).eq.0) then
         cte2=1.d0
        else
         cte2=-1.d0
        endif
        numb=numb+cte2*w(l1,l2,ltot,m1,m2)*funcr(ntot,ltot)
      enddo
      tensor=cte*numb
      if (abs(tensor).lt.1d-20) tensor=0d0
c
      return
      end function
c
c-----------------------------------------------------------------------
c
      function T2(l1,m1,l2,m2) result(RT2)

      use datos, only : R, pi
      integer, intent(in) :: l1, m1, l2, m2
      real*8 :: cte, RT2 

      if (mod(l1,2).eq.0) then ! l1 is even
       cte=4.d0*pi*sqrt((2.d0*l1+1.d0)*(2.d0*l2+1.d0))
      else ! l1 is odd
       cte=-4.d0*pi*sqrt((2.d0*l1+1.d0)*(2.d0*l2+1.d0))
      endif
      RT2=cte*R**(-l1-l2-1)*w(l1,l2,l1+l2,m1,m2)*Delta(l1,l2)
      if (abs(RT2).lt.1d-20) RT2=0d0

      end function T2
c
c-----------------------------------------------------------------------
c
      real*8 function Delta(l1,l2)
   
      integer :: l1, l2

c.....LOCAL VARIABLES:
      integer :: ltot, l1t, l2t

      ltot = l1 +l2
      l1t = 2*l1 + 1
      l2t = 2*l2 + 1
      if (mod(ltot,2).eq.0) then ! ltot is even
       Delta=factorial(2*ltot)*factorial(l1)*factorial(l2)
     & /(factorial(ltot)*factorial(l1t)*factorial(l2t))
      else
       Delta=-factorial(2*ltot)*factorial(l1)*factorial(l2)
     & /(factorial(ltot)*factorial(l1t)*factorial(l2t)) 
      endif
 
      return
      end function
c
c-----------------------------------------------------------------------
c
      real*8 function w(l1,l2,ltot,m1,m2)

      use datos, only : slm,il0
      integer :: l1, l2, ltot
      integer :: m1, m2

c.....LOCAL VARIABLES:
      integer :: mtot

      w=0d0 ! Initializations
      do mtot=-ltot,ltot
        w=w+rgaunt(l1,m1,l2,m2,ltot,mtot)*slm(il0(ltot)+mtot)
      enddo
c
      return
      end function
c
c-----------------------------------------------------------------------
c
      real*8 function funcr(ntot,ltot)

      use datos, only : sigmtot
      integer :: ntot, ltot

c.....LOCAL VARIABLES:
      real*8 :: sigmd, alpha, lgpol

      sigmd=sigmtot*sigmtot
      alpha=ltot+0.5d0
      if (ntot.eq.0) then
        funcr=2.d0**ltot*gamm(alpha,sigmd)
      else
        call laguerre_func (ntot-1,alpha,sigmd,lgpol)
        funcr=lgpol
      endif
c
      return
      end function
c
c-----------------------------------------------------------------------
c
      function T(la,ka,lb,kb) result(RT)

      use datos, only : R
      integer, intent(in) :: la,ka,lb,kb
      real*8 :: RT

      RT=R**(-la-lb-1)*N(la,ka)*N(lb,kb)*Ttilda(la,ka,lb,kb)
      if (abs(RT).lt.1d-20) RT=0d0

      end function T
c
c-----------------------------------------------------------------------
c
      function N(la,ka)

      integer :: la, ka
      real*8 :: N

      N=sqrt(factorial(la-abs(ka))/factorial(la+abs(ka)))
      end function N
c
c-----------------------------------------------------------------------
c
      recursive function Ttilda(la,ka,lb,kb) result(RTtilda)

      use datos, only : tra,trb,c,TtildaList,TtildaLabel
      integer, intent(in) :: la,ka,lb,kb
      real*8 :: RTtilda
      integer :: i,j

      RTtilda=0
      If (la.ge.0.and.lb.ge.0) then
       If (TtildaLabel(la,ka,lb,kb)==0) then
        if (la<abs(ka) .or. lb<abs(kb)) then
         RTtilda=0
        else
         if (lb==0) then
         !case of simple formula (lb=0)
          if (la==0) then
          ! end of recursion
           RTtilda=1
          else
           RTtilda=-(la-1)/la*Ttilda(la-2,ka,lb,kb)
           do i=1,3
            RTtilda=RTtilda+(2*la-1)/la*M(i,ka)*tra(i)*Ttilda(la-1,
     &            Eta(i,ka),lb,kb)
           enddo
          endif
         else
         !case when lb><0
          RTtilda=Ttilda(la-2,ka,lb,kb)-(2*la+lb-1)/lb*Ttilda(la,ka,lb-2
     &     ,kb)
          do j=1,3
           RTtilda=RTtilda+(2*la+2*lb-1)/lb*M(j,kb)*trb(j)*Ttilda(la,ka,
     &          lb-1,Eta(j,kb))
           do i=1,3
            if (c(i,j).ne.0d0) then
             RTtilda=RTtilda+(2*la-1)/lb*M(i,ka)*M(j,kb)*c(i,j)*
     &            Ttilda(la-1,Eta(i,ka),lb-1,Eta(j,kb))
            endif
           enddo
          enddo
         endif
        endif
        TtildaList(la,ka,lb,kb)=RTtilda
        TtildaLabel(la,ka,lb,kb)=1
       else
        RTtilda=TtildaList(la,ka,lb,kb)
       endif
      endif
      end function Ttilda
c
c-----------------------------------------------------------------------
c
      function M(mu,ka)

      integer :: mu,ka
      real*8 :: M

      ! mu equal to 1 if x, to 2 if y, to 3 if z
      if (mu==1) then
       if (ka==0 .or. ka==-1) then
        M=0
       else
        if (ka==1) then
         M=sqrt(2D0)
        else
         M=abs(ka)
        endif
       endif
      else
       if (mu==2) then
        if (ka==0 .or. ka==1) then
         M=0
        else
         if (ka==-1) then
          M=sqrt(2D0)
         else
          M=-ka
         endif
        endif
       else
         M=1
       endif
      endif
      end function M
c
c-----------------------------------------------------------------------
c
      function Eta(mu,ka)

      integer:: mu,ka
      integer :: Eta

      ! mu equal to 1 if x, to 2 if y, to 3 if z
      if (mu==1) then
       if (ka==0 .or. ka==-1 .or. ka==1) then
        Eta=0
       else
        Eta=sign(abs(ka)-1,ka)
       endif
      else
       if (mu==2) then
        if (ka==0 .or. ka==1 .or. ka==-1) then
         Eta=0
        else
         Eta=sign(abs(ka)-1,-ka)
        endif
       else
        if (mu==3) then
         Eta=ka
        endif
       endif
      endif
      end function Eta
c
c-----------------------------------------------------------------------
c
      function ratval(x,cof,mm,kk)
c     Given 'mm', 'kk' and 'cof(1:mm+kk+1)', evaluate and return the
c     rational function (cof(1)+cof(2)*x+...+cof(mm+1)*x**mm)/(1
c     +cof(mm+2)*x+...+cof(mm+kk+1)*x**kk).
c     W. H. Press, S. A. Teukolsky, W. T. Vetterling, B. P. Flanney,
c     "Numerical Recipes in Fortran", 2 edition (1994).
      integer :: kk,mm
      real*8 :: ratval,x,cof(mm+kk+1)
c.....LOCAL VARIABLES:
      integer :: j
      real*8 :: sumd,sumn
      sumn=cof(mm+1)
      do j=mm,1,-1
        sumn=sumn*x+cof(j) !Horner's rule
      enddo
      sumd=0.d0
      do j=mm+kk+1,mm+2,-1
        sumd=(sumd+cof(j))*x !Horner's rule
      enddo
      ratval=sumn/(1.d0+sumd)
c
      return
      end function
c
c-----------------------------------------------------------------------
c
      subroutine ludcmp(a,n,indx)
c     Given a matrix 'a(1:n,1:n)' this routine replaces it by the LU
c     descomposition of a rowwise permutation of itself. 'a' and 'n' are
c     input. 'a' is output, arranged as in equation (2.3.14); 'indx(n)'
c     is an output vector that records the row permutation effected by
c     the partial pivoting. This routine is used in combination with
c     'lubksb' to solve linear equations or invert matrix.
c     W. H. Press, S. A. Teukolsky, W. T. Vetterling, B. P. Flanney,
c     "Numerical Recipes in Fortran", 2 edition (1994).
      use datos, only : faterr
      integer :: n,indx(n)
      real*8 :: a(n,n)
c.....LOCAL VARIABLES:
      real*8 :: TINY
      parameter (TINY=1.0e-20) !A small number
      integer :: i,imax,j,k
      real*8 :: aamax,dum,sum,vv(n) !'vv' stores the implicit scaling
                                       !of each row
c     Loop over rows to get the implicit scaling information
      do i=1,n
        aamax=0.d0
        do j=1,n
          if (abs(a(i,j)).gt.aamax) aamax=abs(a(i,j))
        enddo
c       No nonzero largest element
        if (aamax.eq.0.d0)
     &   call error ('ludcmp','singular matrix in ludcmp', faterr)
c       Save the scaling
        vv(i)=1.d0/aamax
      enddo
c     This is the loop over columns of Crout's method
      do j=1,n
c       This is the equation (2.3.12) except for i=j
        do i=1,j-1
          sum=a(i,j)
          do k=1,i-1
            sum=sum-a(i,k)*a(k,j)
          enddo
          a(i,j)=sum
        enddo
c       Initilaize for search for largest pivot element
        aamax=0.d0
c       This is i=j of equation (2.3.12) and i=j+1,..,N of equation
c       (2.3.13)
        do i=j,n
          sum=a(i,j)
          do k=1,j-1
            sum=sum-a(i,k)*a(k,j)
          enddo
          a(i,j)=sum
c         Figure of merit for the pivot
          dum=vv(i)*abs(sum)
c         Is it better than the best so far?
          if (dum.ge.aamax) then
            imax=i
            aamax=dum
          endif
        enddo
c       Do we need to interchange rows?
        if (j.ne.imax) then
c         Yes, do so...
          do k=1,n
            dum=a(imax,k)
            a(imax,k)=a(j,k)
            a(j,k)=dum
          enddo
c         ...and interchange the scale factor
          vv(imax)=vv(j)
        endif
        indx(j)=imax
        if (a(j,j).eq.0.d0) a(j,j)=TINY
c       Now, finally, divide by the pivot element
        if (j.ne.n) then
          dum=1.d0/a(j,j)
          do i=j+1,n
            a(i,j)=a(i,j)*dum
          enddo
        endif
c     Go back for the next column in the reduction
      enddo
c
      return
      end subroutine
c
c-----------------------------------------------------------------------
c
      subroutine lubksb(a,n,indx,b)
c     Solves the set of n linear equations A*X=B. Here 'a' is input, not
c     as the matrix A but rather as its LU descomposition, determied by
c     the routine 'ludcmp'. 'indx' is input as the permutation vector
c     returned by 'ludcmp'. 'b(1:n)' is input as the right-hnad side
c     vector B, and returns with the solution vector X. 'a' and 'b' are
c     not modified by this routine and can be left in place for
c     sucessive calls with different right-hand sides 'b'. This routine
c     takes into account the possibility that 'b' will begin with many
c     zero elements, so it is efficient for use in matrix inversion.
c     W. H. Press, S. A. Teukolsky, W. T. Vetterling, B. P. Flanney,
c     "Numerical Recipes in Fortran", 2 edition (1994).
      integer :: n,indx(n)
      real*8 :: a(n,n),b(n)
c.....LOCAL VARIABLES:
      integer :: i,ii,j,ll
      real*8 :: sum
c     When ii is set to a positive value, it will become the index of
c     the first nonvanishing element of b. We now do the forward
c     substitution, equation (2.3.6). The only new wrinkle is to
c     unscramble the permutation as we go
      ii=0
      do i=1,n
        ll=indx(i)
        sum=b(ll)
        b(ll)=b(i)
        if (ii.ne.0) then
          do j=ii,i-1
            sum=sum-a(i,j)*b(j)
          enddo
        else if (sum.ne.0.d0) then
c       A nonzero element was encountered, so from now on we will have
c       to do the sums in the loop above
          ii=i
        endif
        b(i)=sum
      enddo
c     Now we do the backsubstitution, equation (2.3.7)
      do i=n,1,-1
        sum=b(i)
        do j=i+1,n
          sum=sum-a(i,j)*b(j)
        enddo
c       Store a component of the solution vector X
        b(i)=sum/a(i,i)
      enddo
c
      return
      end subroutine
c
c-----------------------------------------------------------------------
c
      subroutine mprove (a,alud,n,indx,b,x)
c     Improves a solution vector 'x(1:n)' of the linear set of equations
c     A*X=B. The matrix 'a(1:n,1:n)', and the vectors 'b(1:n)' and
c     'x(1:n)' are input, as is the dimension 'n'. Also input 'alud',
c     the LU descomposition of 'a' as returned by 'ludcmp', and the
c     vector 'indx' also returned by that routine. On output, only
c     'x(1:n)' is modified, to an improved set of values.
c     W. H. Press, S. A. Teukolsky, W. T. Vetterling, B. P. Flanney,
c     "Numerical Recipes in Fortran", 2 edition (1994).
      integer :: n,indx(n)
      real*8 :: a(n,n),alud(n,n),b(n),x(n)
c.....LOCAL VARIABLES:
      integer :: i,j
      real*8 :: r(n)
      real*8 :: sdp
c     Calculate the right-hand side, accumulating the residual
      do i=1,n
        sdp=-b(i)
        do j=1,n
          sdp=sdp+a(i,j)*x(j)
        enddo
        r(i)=sdp
      enddo
c     Solve for the error term, and subtract it from the old solution
      call lubksb (alud,n,indx,r)
      do i=1,n
        x(i)=x(i)-r(i)
      enddo
c
      return
      end subroutine
c
c-------------------------------------------------------------------------
      subroutine rotrecur (R1, Rold, Rnew, l)
c
c.....rotrecur - obtain Wigner rotation matrix R^l from R^(l-1) and R^1
c
c     Routine based on the recursive algorithm by Ivanic and Ruedenberg,
c     J. Phys. Chem. 100, 6342-47 (1996).
c
c.....IMPORTANT NOTE: Ivanic and Ruedenberg defined the coordinate
c     transformation matrix acting on row vectors, and obtained the
c     rotation matrices acting on row vectors, but considering that
c     they should give the RSH value in the rotated point Slm(\hat{R}x);
c     since we want the rotation matrices (still acting on row vectors)
c     for the rotation of the functions,
c       \hat{R} Slm(x) = Slm(R^{-1}x),
c     which is the transpose of the above one, and we work with
c     coordinate transformation matrices acting on column vectors, again
c     the transpose of Ivanic and Ruedenberg definition, we can just use
c     the same algorithm and the double transpose will give our desired
c     convention as output; thus we will obtain:
c       \hat{R} S_lm2(x) = S_lm2(R^{-1}x) = \sum_m1 S_lm1(x) R^l_m1m2
c     We will assume that l>1, loops like do i = 2, 1 should not
c     execute, and do i = 2, 2 should run one trip in order for the l=2
c     and l=3 cases to give the correct answer (this is the standard
c     Fortran 77 behavior).
c
c
c.....Parameters:
c
c     R1(,) ........... rotation matrix for L=1
c     Rold(,) ......... rotation matrix for L=l-1
c     Rnew(,) ......... rotation matrix for L=l
c     l ............... angular momentum L of the required matrix
c
      real*8            R1(-1:1,-1:1), Rold(1-l:l-1,1-l:l-1)
      real*8            Rnew(-l:l,-l:l)
c
      real*8            u, v, w, uu, vv, ww, den
      integer           iu, iv, iw
c
c.....loop over all values of |m1| and |m2|
c
c.....m1 = 0
c       m2 = 0
c
      v = - sqrt((l-1)/(2d0*l))
      uu = R1(0,0) * Rold(0,0)
      vv = R1(1,0) * Rold(1,0) + R1(-1,0) * Rold(-1,0)
*     u = 1,   ww = 0
      Rnew(0,0) = uu + v * vv
c
c.....m1 = 0
c       0 < m2 < l
c
      iu = l*l
      iv = 2*l*(l-1)
      do m2 = 1, l-1
        den = (l-m2) * (l+m2)
        u = sqrt(iu/den)
        v = -0.5d0 * sqrt(iv/den)
        uu = R1(0,0) * Rold(0,m2)
        vv = R1(1,0) * Rold(1,m2) + R1(-1,0) * Rold(-1,m2)
*       ww = 0
        Rnew(0,m2) = u * uu + v * vv
        uu = R1(0,0) * Rold(0,-m2)
        vv = R1(1,0) * Rold(1,-m2) + R1(-1,0) * Rold(-1,-m2)
        Rnew(0,-m2) = u * uu + v * vv
      enddo
c
c.....m1 = 0
c       m2 = l
c
      u = sqrt(l/(2d0*(l+l-1)))
      v = -0.5d0 * sqrt((l-1)/(l+l-1d0))
      uu = R1(0,1) * Rold(0,l-1) - R1(0,-1) * Rold(0,1-l)
      vv = R1(1,1) * Rold(1,l-1) - R1(1,-1) * Rold(1,1-l) +
     &     R1(-1,1) * Rold(-1,l-1) - R1(-1,-1) * Rold(-1,1-l)
*     ww = 0
      Rnew(0,l) = u * uu + v * vv
      uu = R1(0,1) * Rold(0,1-l) + R1(0,-1) * Rold(0,l-1)
      vv = R1(1,1) * Rold(1,1-l) + R1(1,-1) * Rold(1,l-1) +
     &     R1(-1,1) * Rold(-1,1-l) + R1(-1,-1) * Rold(-1,l-1)
      Rnew(0,-l) = u * uu + v * vv
c
c.....m1 = 1
c       m2 = 0
c
      iu = (l-1) * (l+1)
*     sqrt(2)*v, coming from vv
      iv = 2 * l * (l+1)
      iw = (l-2) * (l-1)
*
      den = l * l
      u = sqrt(iu/den)
      v = 0.5d0 * sqrt(iv/den)
      uu = R1(0,0) * Rold(1,0)
      vv = R1(1,0) * Rold(0,0)
      Rnew(1,0) = u * uu + v * vv
      uu = R1(0,0) * Rold(-1,0)
      vv = R1(-1,0) * Rold(0,0)
      Rnew(-1,0) = u * uu + v * vv
      if (iw.ne.0) then
        w = -0.5d0 * sqrt(iw/den)
        ww = R1(1,0) * Rold(2,0) + R1(-1,0) * Rold(-2,0)
        Rnew(1,0) = Rnew(1,0) + w * ww
        ww = R1(1,0) * Rold(-2,0) - R1(-1,0) * Rold(2,0)
        Rnew(-1,0) = Rnew(-1,0) + w * ww
      endif
c
c.....m1 = 1
c       0 < m2 < l
c
      do m2 = 1, l-1
        den = (l-m2) * (l+m2)
        u = sqrt(iu/den)
        v = 0.5d0 * sqrt(iv/den)
        uu = R1(0,0) * Rold(1,m2)
        vv = R1(1,0) * Rold(0,m2)
        Rnew(1,m2) = u * uu + v * vv
        uu = R1(0,0) * Rold(1,-m2)
        vv = R1(1,0) * Rold(0,-m2)
        Rnew(1,-m2) = u * uu + v * vv
        uu = R1(0,0) * Rold(-1,m2)
        vv = R1(-1,0) * Rold(0,m2)
        Rnew(-1,m2) = u * uu + v * vv
        uu = R1(0,0) * Rold(-1,-m2)
        vv = R1(-1,0) * Rold(0,-m2)
        Rnew(-1,-m2) = u * uu + v * vv
        if (iw.ne.0) then
          w = -0.5d0 * sqrt(iw/den)
          ww = R1(1,0) * Rold(2,m2) + R1(-1,0) * Rold(-2,m2)
          Rnew(1,m2) = Rnew(1,m2) + w * ww
          ww = R1(1,0) * Rold(2,-m2) + R1(-1,0) * Rold(-2,-m2)
          Rnew(1,-m2) = Rnew(1,-m2) + w * ww
          ww = R1(1,0) * Rold(-2,m2) - R1(-1,0) * Rold(2,m2)
          Rnew(-1,m2) = Rnew(-1,m2) + w * ww
          ww = R1(1,0) * Rold(-2,-m2) - R1(-1,0) * Rold(2,-m2)
          Rnew(-1,-m2) = Rnew(-1,-m2) + w * ww
        endif
      enddo
c
c.....m1 = 1
c       m2 = l
c
      den = 2 * l * (l+l-1)
      u = sqrt(iu/den)
      v = 0.5d0 * sqrt(iv/den)
      uu = R1(0,1) * Rold(1,l-1) - R1(0,-1) * Rold(1,1-l)
      vv = R1(1,1) * Rold(0,l-1) - R1(1,-1) * Rold(0,1-l)
      Rnew(1,l) = u * uu + v * vv
      uu = R1(0,1) * Rold(1,1-l) + R1(0,-1) * Rold(1,l-1)
      vv = R1(1,1) * Rold(0,1-l) + R1(1,-1) * Rold(0,l-1)
      Rnew(1,-l) = u * uu + v * vv
      uu = R1(0,1) * Rold(-1,l-1) - R1(0,-1) * Rold(-1,1-l)
      vv = R1(-1,1) * Rold(0,l-1) - R1(-1,-1) * Rold(0,1-l)
      Rnew(-1,l) = u * uu + v * vv
      uu = R1(0,1) * Rold(-1,1-l) + R1(0,-1) * Rold(-1,l-1)
      vv = R1(-1,1) * Rold(0,1-l) + R1(-1,-1) * Rold(0,l-1)
      Rnew(-1,-l) = u * uu + v * vv
      if (iw.ne.0) then
        w = -0.5d0 * sqrt(iw/den)
        ww = R1(1,1) * Rold(2,l-1) - R1(1,-1) * Rold(2,1-l) +
     &       R1(-1,1) * Rold(-2,l-1) - R1(-1,-1) * Rold(-2,1-l)
        Rnew(1,l) = Rnew(1,l) + w * ww
        ww = R1(1,1) * Rold(2,1-l) + R1(1,-1) * Rold(2,l-1) +
     &       R1(-1,1) * Rold(-2,1-l) + R1(-1,-1) * Rold(-2,l-1)
        Rnew(1,-l) = Rnew(1,-l) + w * ww
        ww = R1(1,1) * Rold(-2,l-1) - R1(1,-1) * Rold(-2,1-l) -
     &       R1(-1,1) * Rold(2,l-1) + R1(-1,-1) * Rold(2,1-l)
        Rnew(-1,l) = Rnew(-1,l) + w * ww
        ww = R1(1,1) * Rold(-2,1-l) + R1(1,-1) * Rold(-2,l-1) -
     &       R1(-1,1) * Rold(2,1-l) - R1(-1,-1) * Rold(2,l-1)
        Rnew(-1,-l) = Rnew(-1,-l) + w * ww
      endif
c
c.....1 < m1 < l
c       m2 = 0
c
      do m1 = 2, l-1
        iu = (l-m1) * (l+m1)
        iv = (l-1+m1) * (l+m1)
        iw = (l-1-m1) * (l-m1)
*
        den = l * l
        u = sqrt(iu/den)
        v = 0.5d0 * sqrt(iv/den)
        uu = R1(0,0) * Rold(m1,0)
        vv = R1(1,0) * Rold(m1-1,0) - R1(-1,0) * Rold(1-m1,0)
        Rnew(m1,0) = u * uu + v * vv
        uu = R1(0,0) * Rold(-m1,0)
        vv = R1(1,0) * Rold(1-m1,0) + R1(-1,0) * Rold(m1-1,0)
        Rnew(-m1,0) = u * uu + v * vv
        if (iw.ne.0) then
          w = -0.5d0 * sqrt(iw/den)
          ww = R1(1,0) * Rold(m1+1,0) + R1(-1,0) * Rold(-m1-1,0)
          Rnew(m1,0) = Rnew(m1,0) + w * ww
          ww = R1(1,0) * Rold(-m1-1,0) - R1(-1,0) * Rold(m1+1,0)
          Rnew(-m1,0) = Rnew(-m1,0) + w * ww
        endif
c
c.....1 < m1 < l
c       0 < m2 < l
c
        do m2 = 1, l-1
          den = (l-m2) * (l+m2)
          u = sqrt(iu/den)
          v = 0.5d0 * sqrt(iv/den)
          uu = R1(0,0) * Rold(m1,m2)
          vv = R1(1,0) * Rold(m1-1,m2) - R1(-1,0) * Rold(1-m1,m2)
          Rnew(m1,m2) = u * uu + v * vv
          uu = R1(0,0) * Rold(m1,-m2)
          vv = R1(1,0) * Rold(m1-1,-m2) - R1(-1,0) * Rold(1-m1,-m2)
          Rnew(m1,-m2) = u * uu + v * vv
          uu = R1(0,0) * Rold(-m1,m2)
          vv = R1(1,0) * Rold(1-m1,m2) + R1(-1,0) * Rold(m1-1,m2)
          Rnew(-m1,m2) = u * uu + v * vv
          uu = R1(0,0) * Rold(-m1,-m2)
          vv = R1(1,0) * Rold(1-m1,-m2) + R1(-1,0) * Rold(m1-1,-m2)
          Rnew(-m1,-m2) = u * uu + v * vv
          if (iw.ne.0) then
            w = -0.5d0 * sqrt(iw/den)
            ww = R1(1,0) * Rold(m1+1,m2) + R1(-1,0) * Rold(-m1-1,m2)
            Rnew(m1,m2) = Rnew(m1,m2) + w * ww
            ww = R1(1,0) * Rold(m1+1,-m2) + R1(-1,0) * Rold(-m1-1,-m2)
            Rnew(m1,-m2) = Rnew(m1,-m2) + w * ww
            ww = R1(1,0) * Rold(-m1-1,m2) - R1(-1,0) * Rold(m1+1,m2)
            Rnew(-m1,m2) = Rnew(-m1,m2) + w * ww
            ww = R1(1,0) * Rold(-m1-1,-m2) - R1(-1,0) * Rold(m1+1,-m2)
            Rnew(-m1,-m2) = Rnew(-m1,-m2) + w * ww
          endif
        enddo
c
c.....1 < m1 < l
c       m2 = l
c
        den = 2 * l * (l+l-1)
        u = sqrt(iu/den)
        v = 0.5d0 * sqrt(iv/den)
        uu = R1(0,1) * Rold(m1,l-1) - R1(0,-1) * Rold(m1,1-l)
        vv = R1(1,1) * Rold(m1-1,l-1) - R1(1,-1) * Rold(m1-1,1-l) -
     &       R1(-1,1) * Rold(1-m1,l-1) + R1(-1,-1) * Rold(1-m1,1-l)
        Rnew(m1,l) = u * uu + v * vv
        uu = R1(0,1) * Rold(m1,1-l) + R1(0,-1) * Rold(m1,l-1)
        vv = R1(1,1) * Rold(m1-1,1-l) + R1(1,-1) * Rold(m1-1,l-1) -
     &       R1(-1,1) * Rold(1-m1,1-l) - R1(-1,-1) * Rold(1-m1,l-1)
        Rnew(m1,-l) = u * uu + v * vv
        uu = R1(0,1) * Rold(-m1,l-1) - R1(0,-1) * Rold(-m1,1-l)
        vv = R1(1,1) * Rold(1-m1,l-1) - R1(1,-1) * Rold(1-m1,1-l) +
     &       R1(-1,1) * Rold(m1-1,l-1) - R1(-1,-1) * Rold(m1-1,1-l)
        Rnew(-m1,l) = u * uu + v * vv
        uu = R1(0,1) * Rold(-m1,1-l) + R1(0,-1) * Rold(-m1,l-1)
        vv = R1(1,1) * Rold(1-m1,1-l) + R1(1,-1) * Rold(1-m1,l-1) +
     &       R1(-1,1) * Rold(m1-1,1-l) + R1(-1,-1) * Rold(m1-1,l-1)
        Rnew(-m1,-l) = u * uu + v * vv
        if (iw.ne.0) then
          w = -0.5d0 * sqrt(iw/den)
          ww = R1(1,1) * Rold(m1+1,l-1) - R1(1,-1) * Rold(m1+1,1-l) +
     &         R1(-1,1) * Rold(-1-m1,l-1) - R1(-1,-1) * Rold(-1-m1,1-l)
          Rnew(m1,l) = Rnew(m1,l) + w * ww
          ww = R1(1,1) * Rold(m1+1,1-l) + R1(1,-1) * Rold(m1+1,l-1) +
     &         R1(-1,1) * Rold(-1-m1,1-l) + R1(-1,-1) * Rold(-1-m1,l-1)
          Rnew(m1,-l) = Rnew(m1,-l) + w * ww
          ww = R1(1,1) * Rold(-1-m1,l-1) - R1(1,-1) * Rold(-1-m1,1-l) -
     &         R1(-1,1) * Rold(m1+1,l-1) + R1(-1,-1) * Rold(m1+1,1-l)
          Rnew(-m1,l) = Rnew(-m1,l) + w * ww
          ww = R1(1,1) * Rold(-1-m1,1-l) + R1(1,-1) * Rold(-1-m1,l-1) -
     &         R1(-1,1) * Rold(m1+1,1-l) - R1(-1,-1) * Rold(m1+1,l-1)
          Rnew(-m1,-l) = Rnew(-m1,-l) + w * ww
        endif
      enddo
c
c.....m1 = l
c       m2 = 0
c
*     u, w = 0
      v = sqrt((l+l-1d0)/(l+l))
      vv = R1(1,0) * Rold(l-1,0) - R1(-1,0) * Rold(1-l,0)
      Rnew(l,0) = v * vv
      vv = R1(1,0) * Rold(1-l,0) + R1(-1,0) * Rold(l-1,0)
      Rnew(-l,0) = v * vv
c
c.....m1 = l
c       0 < m2 < l
c
      iv = (l+l) * (l+l-1)
      do m2 = 1, l-1
        den = (l-m2) * (l+m2)
        v = 0.5d0 * sqrt(iv/den)
        vv = R1(1,0) * Rold(l-1,m2) - R1(-1,0) * Rold(1-l,m2)
        Rnew(l,m2) = v * vv
        vv = R1(1,0) * Rold(l-1,-m2) - R1(-1,0) * Rold(1-l,-m2)
        Rnew(l,-m2) = v * vv
        vv = R1(1,0) * Rold(1-l,m2) + R1(-1,0) * Rold(l-1,m2)
        Rnew(-l,m2) = v * vv
        vv = R1(1,0) * Rold(1-l,-m2) + R1(-1,0) * Rold(l-1,-m2)
        Rnew(-l,-m2) = v * vv
      enddo
c
c.....m1 = l
c       m2 = l
c
      v = 0.5d0
      vv = R1(1,1) * Rold(l-1,l-1) - R1(1,-1) * Rold(l-1,1-l) -
     &     R1(-1,1) * Rold(1-l,l-1) + R1(-1,-1) * Rold(1-l,1-l)
      Rnew(l,l) = v * vv
      vv = R1(1,1) * Rold(l-1,1-l) + R1(1,-1) * Rold(l-1,l-1) -
     &     R1(-1,1) * Rold(1-l,1-l) - R1(-1,-1) * Rold(1-l,l-1)
      Rnew(l,-l) = v * vv
      vv = R1(1,1) * Rold(1-l,l-1) - R1(1,-1) * Rold(1-l,1-l) +
     &     R1(-1,1) * Rold(l-1,l-1) - R1(-1,-1) * Rold(l-1,1-l)
      Rnew(-l,l) = v * vv
      vv = R1(1,1) * Rold(1-l,1-l) + R1(1,-1) * Rold(1-l,l-1) +
     &     R1(-1,1) * Rold(l-1,1-l) + R1(-1,-1) * Rold(l-1,l-1)
      Rnew(-l,-l) = v * vv
c
c.....routine ends
c
      return
      end subroutine

**************************************************************************

      subroutine calc_newmm(newmm,oldmm,rad)

      use datos, only: ltot 

*     ***************************************************
*     PURPOSE : CALCULATION OF THE NEW SHIFTED MULTIPOLES
*     (See Stone : p.32-35)
*     ***************************************************

      real*8 :: newmm(0:ltot,0:ltot,2)
      real*8 :: oldmm(0:ltot,0:ltot,2)
      real*8 :: rad
c.....LOCAL VARIABLES:
      real*8 :: add
      integer :: imax, jmax, s
      integer :: l1, l2, k1, k2, i, j

      do l1=0,ltot
        do k1=0,l1
           if (k1.eq.0) then
              imax=1
           else
              imax=2
           endif
           do i=1,imax
              newmm(l1,k1,i)=0d0
              do l2=0,l1
                 do k2=-l2,l2
                    if (k2.eq.0) then
                       jmax=1
                    else
                       jmax=2
                    endif
                    do j=1,jmax
                       if (k2.lt.0.and.j.eq.2) then
                           s=-1
                       else
                           s=1
                       endif
                       add=s*oldmm(l2,abs(k2),j)
     &                     *Wfac(l1,l2,k1,k2,i,j,rad)
                       newmm(l1,k1,i)=newmm(l1,k1,i)+add
                    enddo
                 enddo
              enddo
           enddo
        enddo
      enddo

      return
      end subroutine

**************************************************************************

      subroutine calc_tnewmm(newmm,oldmm,rad,l)


*     ***************************************************
*     PURPOSE : CALCULATION OF THE NEW SHIFTED MULTIPOLES
*     (See Stone : p.32-35)
*     ***************************************************

      integer :: l  
      real*8  :: newmm(0:l,0:l,2)
      real*8  :: oldmm(0:l,0:l,2)
      real*8  :: rad
c.....LOCAL VARIABLES:
      real*8 :: add
      integer :: imax, jmax, s
      integer :: l1, l2, k1, k2, i, j

      do l1=0,l
        do k1=0,l1
           if (k1.eq.0) then
              imax=1
           else
              imax=2
           endif
           do i=1,imax
              newmm(l1,k1,i)=0d0
              do l2=0,l1
                 do k2=-l2,l2
                    if (k2.eq.0) then
                       jmax=1
                    else
                       jmax=2
                    endif
                    do j=1,jmax
                       if (k2.lt.0.and.j.eq.2) then
                           s=-1
                       else
                           s=1
                       endif
                       add=s*oldmm(l2,abs(k2),j)
     &                     *Wfact(l1,l2,k1,k2,i,j,rad)
                       newmm(l1,k1,i)=newmm(l1,k1,i)+add
                    enddo
                 enddo
              enddo
           enddo
        enddo
      enddo

      return
      end subroutine

*************************************************************************
      
      real*8 function Wfac(l1,l2,k1,k2,i,j,rad)

*     ***********************************************
*     PURPOSE : CALCULATE THE W FUNCTION (Stone p.34)
*     FOR A MM SHIFT
*     ***********************************************
      use datos, only : faterr, numbl 
      real*8 :: rad
      integer :: l1, l2, k1, k2, i, j
c.....LOCAL VARIABLES:
      real*8 :: R, bin1, bin2, alpha
      real*8 :: coef

      if (l1.lt.l2) then
       Wfac=0d0
      else

       bin1=binom(l1+k1,l2+k2)
       bin2=binom(l1-k1,l2-k2)

       alpha=Fb(k1)/(2*Fb(k2)*Fb(k1-k2))

       if ((l1-l2).eq.0.and.rad.eq.0d0) then
         coef=numbl(0)
       else
         coef=numbl(l1-l2)*rad**(l1-l2)
       endif

       if ((l1-l2).ge.abs(k1-k2)) then
         if (i.eq.j) then
            if (k1.lt.k2) then
                R=coef*Sc(l1-l2,k2-k1)
            else
                R=coef*Sc(l1-l2,k1-k2)
            endif
         else
            if (k1.lt.k2) then
               R=coef*Ss(l1-l2,k2-k1)
               R=-R
            else
               R=coef*Ss(l1-l2,k1-k2)
            endif
            if (i.eq.1) R=-R 
         endif
       else
         R=0d0
       endif

       Wfac=sqrt(bin1*bin2)*alpha*R
       
       if (abs(Wfac).lt.1d-20) Wfac=0d0

       if (l1-k1.lt.0) 
     &  call error ('Wfac',' l1-k1 < 0', faterr) 
       if (l2-k2.lt.0) 
     &  call error ('Wfac',' l2-k2 < 0', faterr)
      endif

      return
      end function

*************************************************************************

      real*8 function Wfact(l1,l2,k1,k2,i,j,rad)

*     ***********************************************
*     PURPOSE : CALCULATE THE W FUNCTION (Stone p.34)
*     FOR A MM SHIFT
*     ***********************************************
      use datos, only : faterr, numbl
      real*8 :: rad
      integer :: l1, l2, k1, k2, i, j
c.....LOCAL VARIABLES:
      real*8 :: R, bin1, bin2, alpha
      real*8 :: coef

      if (l1.lt.l2) then
       Wfact=0d0
      else

       bin1=binom(l1+k1,l2+k2)
       bin2=binom(l1-k1,l2-k2)

       alpha=Fb(k1)/(2*Fb(k2)*Fb(k1-k2))

       if ((l1-l2).eq.0.and.rad.eq.0d0) then
         coef=numbl(0)
       else
         coef=numbl(l1-l2)*rad**(l1-l2)
       endif

       if ((l1-l2).ge.abs(k1-k2)) then
         if (i.eq.j) then
            if (k1.lt.k2) then
                R=coef*Sct(l1-l2,k2-k1)
            else
                R=coef*Sct(l1-l2,k1-k2)
            endif
         else
            if (k1.lt.k2) then
               R=coef*Sst(l1-l2,k2-k1)
               R=-R
            else
               R=coef*Sst(l1-l2,k1-k2)
            endif
            if (i.eq.1) R=-R
         endif
       else
         R=0d0
       endif

       Wfact=sqrt(bin1*bin2)*alpha*R

       if (abs(Wfact).lt.1d-20) Wfact=0d0

       if (l1-k1.lt.0)
     &  call error ('Wfact',' l1-k1 < 0', faterr)
       if (l2-k2.lt.0)
     &  call error ('Wfact',' l2-k2 < 0', faterr)
      endif

      return
      end function

*******************************************************************

        
      real*8 function binom(n,m)


      integer :: n,m

      if (n.ge.m) then 
          binom=factorial(n)/(factorial(m)*factorial(n-m))
      else
          binom=0d0 
      endif
      
      return
      end function

*******************************************************************

      real*8 function Fb(k)


      integer :: k
c.....LOCAL VARIABLES:
      real*8 :: fac

*     *******************************************************
*     PURPOSE : CALCULATE b TERMS FOR A MM SHIFT (Stone p.34)
*     *******************************************************
      
      if(k.gt.0) then
         fac=1d0
         if (mod(k,2).eq.1) fac=-1d0
         Fb=sqrt(0.5d0)*fac  
      else if (k.eq.0) then
         Fb=0.5d0
      else
         Fb=sqrt(0.5d0)
      endif
      
      return
      end function

*******************************************************************

        real*8 function Sc(l,m)
! computes RSHs 
! m and l are integers: l>=0; 0<=m<=l
      use datos, only : faterr, slm2, il02
      integer :: l, m

      if (m.lt.0) 
     & call error ('Sc','m is a negative integer', faterr)
      if (m.gt.l)
     & call error ('Sc','m is greather than l', faterr) 

      Sc = slm2(il02(l)+m)

      return 
      end function

*****************************************************************************

        real*8 function Ss(l,m)
! computes RSHs
! m and l are integers: l>=0; 0<m<=l
      use datos, only : faterr, slm2, il02
      integer:: l, m

      if (m.lt.0)
     & call error ('Ss','m is a negative integer', faterr)
      if (m.gt.l)
     & call error ('Ss','m is greather than l', faterr)

      if (m.gt.0) then
        Ss = slm2(il02(l)-m)
      else ! m = 0
        Ss = 0d0
      endif

      return
      end function

*****************************************************************************

       real*8 function Sct(l,m)
! computes RSHs 
! m and l are integers: l>=0; 0<=m<=l
      use datos, only : faterr, slm, il0
      integer :: l, m

      if (m.lt.0)
     & call error ('Sct','m is a negative integer', faterr)
      if (m.gt.l)
     & call error ('Sct','m is greather than l', faterr)

      Sct = slm(il0(l)+m)

      return
      end function

*****************************************************************************

        real*8 function Sst(l,m)
! computes RSHs
! m and l are integers: l>=0; 0<m<=l
      use datos, only : faterr, slm, il0
      integer:: l, m

      if (m.lt.0)
     & call error ('Sst','m is a negative integer', faterr)
      if (m.gt.l)
     & call error ('Sst','m is greather than l', faterr)

      if (m.gt.0) then
        Sst = slm(il0(l)-m)
      else ! m = 0
        Sst = 0d0
      endif

      return
      end function

*****************************************************************************

      function arrayk(k) result(Rarrayk)
      
      integer, intent(in) :: k
      integer :: Rarrayk

      if (k>0) then
       Rarrayk=2*k
      else
       Rarrayk=2*abs(k)+1
      endif
      end function arrayk

**************************************************************************************
C From Leonard J. Moss of SLAC:

C Here's a hybrid QuickSort I wrote a number of years ago.  It's
C based on suggestions in Knuth, Volume 3, and performs much better
C than a pure QuickSort on short or partially ordered input arrays.  

      SUBROUTINE SORTRX(N,DATA,INDEX)
C===================================================================
C
C     SORTRX -- SORT, Real input, indeX output
C
C
C     Input:  N     INTEGER
C             DATA  REAL*8
C
C     Output: INDEX INTEGER (DIMENSION N)
C
C This routine performs an in-memory sort of the first N elements of
C array DATA, returning into array INDEX the indices of elements of
C DATA arranged in ascending order.  Thus,
C
C    DATA(INDEX(1)) will be the smallest number in array DATA;
C    DATA(INDEX(N)) will be the largest number in DATA.
C
C The original data is not physically rearranged.  The original order
C of equal input values is not necessarily preserved.
C
C===================================================================
C
C SORTRX uses a hybrid QuickSort algorithm, based on several
C suggestions in Knuth, Volume 3, Section 5.2.2.  In particular, the
C "pivot key" [my term] for dividing each subsequence is chosen to be
C the median of the first, last, and middle values of the subsequence;
C and the QuickSort is cut off when a subsequence has 9 or fewer
C elements, and a straight insertion sort of the entire array is done
C at the end.  The result is comparable to a pure insertion sort for
C very short arrays, and very fast for very large arrays (of order 12
C micro-sec/element on the 3081K for arrays of 10K elements).  It is
C also not subject to the poor performance of the pure QuickSort on
C partially ordered data.
C
C Created:  15 Jul 1986  Len Moss
C
C===================================================================
 
      INTEGER   N,INDEX(N)
      REAL*8      DATA(N)
 
      INTEGER   LSTK(31),RSTK(31),ISTK
      INTEGER   L,R,I,J,P,INDEXP,INDEXT
      REAL*8      DATAP
 
C     QuickSort Cutoff
C
C     Quit QuickSort-ing when a subsequence contains M or fewer
C     elements and finish off at end with straight insertion sort.
C     According to Knuth, V.3, the optimum value of M is around 9.
 
      INTEGER   M
      PARAMETER (M=9)
 
C===================================================================
C
C     Make initial guess for INDEX
 
      DO 50 I=1,N
         INDEX(I)=I
   50    CONTINUE
 
C     If array is short, skip QuickSort and go directly to
C     the straight insertion sort.
 
      IF (N.LE.M) GOTO 900
 
C===================================================================
C
C     QuickSort
C
C     The "Qn:"s correspond roughly to steps in Algorithm Q,
C     Knuth, V.3, PP.116-117, modified to select the median
C     of the first, last, and middle elements as the "pivot
C     key" (in Knuth's notation, "K").  Also modified to leave
C     data in place and produce an INDEX array.  To simplify
C     comments, let DATA[I]=DATA(INDEX(I)).
 
C Q1: Initialize
      ISTK=0
      L=1
      R=N
 
  200 CONTINUE
 
C Q2: Sort the subsequence DATA[L]..DATA[R].
C
C     At this point, DATA[l] <= DATA[m] <= DATA[r] for all l < L,
C     r > R, and L <= m <= R.  (First time through, there is no
C     DATA for l < L or r > R.)
 
      I=L
      J=R
 
C Q2.5: Select pivot key
C
C     Let the pivot, P, be the midpoint of this subsequence,
C     P=(L+R)/2; then rearrange INDEX(L), INDEX(P), and INDEX(R)
C     so the corresponding DATA values are in increasing order.
C     The pivot key, DATAP, is then DATA[P].
 
      P=(L+R)/2
      INDEXP=INDEX(P)
      DATAP=DATA(INDEXP)
 
      IF (DATA(INDEX(L)) .GT. DATAP) THEN
         INDEX(P)=INDEX(L)
         INDEX(L)=INDEXP
         INDEXP=INDEX(P)
         DATAP=DATA(INDEXP)
      ENDIF
 
      IF (DATAP .GT. DATA(INDEX(R))) THEN
         IF (DATA(INDEX(L)) .GT. DATA(INDEX(R))) THEN
            INDEX(P)=INDEX(L)
            INDEX(L)=INDEX(R)
         ELSE
            INDEX(P)=INDEX(R)
         ENDIF
         INDEX(R)=INDEXP
         INDEXP=INDEX(P)
         DATAP=DATA(INDEXP)
      ENDIF
 
C     Now we swap values between the right and left sides and/or
C     move DATAP until all smaller values are on the left and all
C     larger values are on the right.  Neither the left or right
C     side will be internally ordered yet; however, DATAP will be
C     in its final position.
 
  300 CONTINUE
 
C Q3: Search for datum on left >= DATAP
C
C     At this point, DATA[L] <= DATAP.  We can therefore start scanning
C     up from L, looking for a value >= DATAP (this scan is guaranteed
C     to terminate since we initially placed DATAP near the middle of
C     the subsequence).
 
         I=I+1
         IF (DATA(INDEX(I)).LT.DATAP) GOTO 300
 
  400 CONTINUE
 
C Q4: Search for datum on right <= DATAP
C
C     At this point, DATA[R] >= DATAP.  We can therefore start scanning
C     down from R, looking for a value <= DATAP (this scan is guaranteed
C     to terminate since we initially placed DATAP near the middle of
C     the subsequence).
 
         J=J-1
         IF (DATA(INDEX(J)).GT.DATAP) GOTO 400
 
C Q5: Have the two scans collided?
 
      IF (I.LT.J) THEN
 
C Q6: No, interchange DATA[I] <--> DATA[J] and continue
 
         INDEXT=INDEX(I)
         INDEX(I)=INDEX(J)
         INDEX(J)=INDEXT
         GOTO 300
      ELSE
 
C Q7: Yes, select next subsequence to sort
C
C     At this point, I >= J and DATA[l] <= DATA[I] == DATAP <= DATA[r],
C     for all L <= l < I and J < r <= R.  If both subsequences are
C     more than M elements long, push the longer one on the stack and
C     go back to QuickSort the shorter; if only one is more than M
C     elements long, go back and QuickSort it; otherwise, pop a
C     subsequence off the stack and QuickSort it.
 
         IF (R-J .GE. I-L .AND. I-L .GT. M) THEN
            ISTK=ISTK+1
            LSTK(ISTK)=J+1
            RSTK(ISTK)=R
            R=I-1
         ELSE IF (I-L .GT. R-J .AND. R-J .GT. M) THEN
            ISTK=ISTK+1
            LSTK(ISTK)=L
            RSTK(ISTK)=I-1
            L=J+1
         ELSE IF (R-J .GT. M) THEN
            L=J+1
         ELSE IF (I-L .GT. M) THEN
            R=I-1
         ELSE
C Q8: Pop the stack, or terminate QuickSort if empty
            IF (ISTK.LT.1) GOTO 900
            L=LSTK(ISTK)
            R=RSTK(ISTK)
            ISTK=ISTK-1
         ENDIF
         GOTO 200
      ENDIF
 
  900 CONTINUE
 
C===================================================================
C
C Q9: Straight Insertion sort
 
      DO 950 I=2,N
         IF (DATA(INDEX(I-1)) .GT. DATA(INDEX(I))) THEN
            INDEXP=INDEX(I)
            DATAP=DATA(INDEXP)
            P=I-1
  920       CONTINUE
               INDEX(P+1) = INDEX(P)
               P=P-1
               IF (P.GT.0) THEN
                  IF (DATA(INDEX(P)).GT.DATAP) GOTO 920
               ENDIF
            INDEX(P+1) = INDEXP
         ENDIF
  950    CONTINUE
 
C===================================================================
C
C     All done
 
      END SUBROUTINE

c--------------------------------------------------------------------      
      logical function equiv (numb,numb2)

      use datos, only : nop, neq, ncent, ineq
      integer :: numb  ! input
      integer :: numb2 ! output
c     LOCAL VARIABLES
      integer :: i, j, eq(ncent), delta
      logical :: ok
    
      equiv = .false.
      numb2 = numb
      if (numb.lt.0 .or. numb.gt.ncent) return        
      if (nop.gt.1) then ! equivalent atoms
        ok = .false.
        j = 1
        do while (.not.ok .and. j.le.neq)
          ok = numb.eq.ineq(j)
          j = j + 1
        enddo      
        if (ok) then
          equiv = .true.
        else
          do j = 1, ncent
            eq(j) = 0
          enddo
          do i = 1, neq-1
            delta = ineq(i+1) - ineq(i)
            if (delta.gt.1) then
              do j = i+1, i+delta-1
                eq(j) = i
              enddo
            endif
          enddo
          if (ineq(neq).ne.ncent) then
            do j = ineq(neq)+1, ncent
              eq(j) = ineq(neq)
            enddo
          endif
          ok = eq(numb).ne.0
          if (ok) then
            numb2 = eq(numb)
            equiv = .true.
          endif
        endif  
      else ! non-equivalent atoms
        ok = .false.
        j = 1
        do while (.not.ok .and. j.le.neq)
          ok = numb.eq.ineq(j)
          j = j + 1
        enddo 
        if (ok) equiv = .true.     
      endif  

      return
      end function

c
      end module libreria 
